@@ -1,21 +1,27 @@
 parafac <- 
-  function(X,nfac,nstart=10,const=NULL,control=NULL,
-           Bfixed=NULL,Cfixed=NULL,Dfixed=NULL,
-           Bstart=NULL,Cstart=NULL,Dstart=NULL,
-           Bstruc=NULL,Cstruc=NULL,Dstruc=NULL,
-           maxit=500,ctol=1e-4,parallel=FALSE,
-           cl=NULL,output=c("best","all")){
+  function(X, nfac, nstart = 10, const = NULL, control = NULL,
+           Bfixed = NULL, Cfixed = NULL, Dfixed = NULL,
+           Bstart = NULL, Cstart = NULL, Dstart = NULL,
+           Bstruc = NULL, Cstruc = NULL, Dstruc = NULL,
+           maxit = 500, ctol = 1e-4, parallel = FALSE, cl = NULL, 
+           output = c("best", "all"), verbose = FALSE){
     # 3-way or 4-way Parallel Factor Analysis (Parafac)
     # via alternating least squares (ALS) with optional constraints
     # Nathaniel E. Helwig (helwig@umn.edu)
-    # last updated: May 16, 2017
+    # last updated: November 12, 2017
     
     # check 'X' input
     xdim <- dim(X)
     lxdim <- length(xdim)
     if(lxdim<3L | lxdim>4L){stop("Input 'X' must be 3-way or 4-way array")}
-    if(any(is.na(X)) | any(is.nan(X)) | any(is.infinite(X))){stop("Input 'X' cannot contain NA, NaN, or Inf values")}
-    xcx <- sumsq(X)
+    if(any(any(is.nan(X)) | any(is.infinite(X)))){stop("Input 'X' cannot contain NaN or Inf values")}
+    if(any(is.na(X))){
+      missingdata <- TRUE
+      naid <- which(is.na(X))
+    } else {
+      missingdata <- FALSE
+      xcx <- sumsq(X)
+    }
     
     # check 'nfac' and 'nstart' inputs
     nfac <- as.integer(nfac[1])
@@ -98,17 +104,36 @@ parafac <-
       if(parallel){
         nstartlist <- vector("list",nstart)
         nstartlist[1:nstart] <- nfac
-        pfaclist <- parLapply(cl=cl,X=nstartlist,fun="parafac_3way",data=X,xcx=xcx,
-                              const=const,control=control,maxit=maxit,ctol=ctol,
-                              Bfixed=Bfixed,Cfixed=Cfixed,Bstart=Bstart,
-                              Cstart=Cstart,Bstruc=Bstruc,Cstruc=Cstruc)
+        if(missingdata){
+          pfaclist <- parLapply(cl=cl,X=nstartlist,fun="parafac_3wayna",data=X,naid=naid,
+                                const=const,control=control,maxit=maxit,ctol=ctol,
+                                Bfixed=Bfixed,Cfixed=Cfixed,Bstart=Bstart,
+                                Cstart=Cstart,Bstruc=Bstruc,Cstruc=Cstruc)
+        } else {
+          pfaclist <- parLapply(cl=cl,X=nstartlist,fun="parafac_3way",data=X,xcx=xcx,
+                                const=const,control=control,maxit=maxit,ctol=ctol,
+                                Bfixed=Bfixed,Cfixed=Cfixed,Bstart=Bstart,
+                                Cstart=Cstart,Bstruc=Bstruc,Cstruc=Cstruc)
+        } # end if(missingdata)
       } else {
         pfaclist <- vector("list",nstart)
-        for(j in 1:nstart){
-          pfaclist[[j]] <- parafac_3way(data=X,nfac=nfac,xcx=xcx,const=const,control=control,
-                                        maxit=maxit,ctol=ctol,Bfixed=Bfixed,Cfixed=Cfixed,
-                                        Bstart=Bstart,Cstart=Cstart,Bstruc=Bstruc,Cstruc=Cstruc)
-        }
+        if(verbose) pbar <- txtProgressBar(min = 0, max = nstart, style = 3)
+        if(missingdata){
+          for(j in 1:nstart){
+            pfaclist[[j]] <- parafac_3wayna(data=X,nfac=nfac,naid=naid,const=const,control=control,
+                                            maxit=maxit,ctol=ctol,Bfixed=Bfixed,Cfixed=Cfixed,
+                                            Bstart=Bstart,Cstart=Cstart,Bstruc=Bstruc,Cstruc=Cstruc)
+            if(verbose) setTxtProgressBar(pbar, j)
+          }
+        } else {
+          for(j in 1:nstart){
+            pfaclist[[j]] <- parafac_3way(data=X,nfac=nfac,xcx=xcx,const=const,control=control,
+                                          maxit=maxit,ctol=ctol,Bfixed=Bfixed,Cfixed=Cfixed,
+                                          Bstart=Bstart,Cstart=Cstart,Bstruc=Bstruc,Cstruc=Cstruc)
+            if(verbose) setTxtProgressBar(pbar, j)
+          }
+        } # end if(missingdata)
+        if(verbose) close(pbar)
       } # end if(parallel)
     } else if(lxdim==4L){
       # check 'Dfixed', 'Dstart', and 'Dstruc' inputs
@@ -131,31 +156,54 @@ parafac <-
       if(parallel){
         nstartlist <- vector("list",nstart)
         nstartlist[1:nstart] <- nfac
-        pfaclist <- parLapply(cl=cl,X=nstartlist,fun="parafac_4way",data=X,xcx=xcx,
-                              const=const,control=control,maxit=maxit,ctol=ctol,Bfixed=Bfixed,
-                              Cfixed=Cfixed,Dfixed=Dfixed,Bstart=Bstart,Cstart=Cstart,
-                              Dstart=Dstart,Bstruc=Bstruc,Cstruc=Cstruc,Dstruc=Dstruc)
+        if(missingdata){
+          pfaclist <- parLapply(cl=cl,X=nstartlist,fun="parafac_4wayna",data=X,naid=naid,
+                                const=const,control=control,maxit=maxit,ctol=ctol,Bfixed=Bfixed,
+                                Cfixed=Cfixed,Dfixed=Dfixed,Bstart=Bstart,Cstart=Cstart,
+                                Dstart=Dstart,Bstruc=Bstruc,Cstruc=Cstruc,Dstruc=Dstruc)
+        } else {
+          pfaclist <- parLapply(cl=cl,X=nstartlist,fun="parafac_4way",data=X,xcx=xcx,
+                                const=const,control=control,maxit=maxit,ctol=ctol,Bfixed=Bfixed,
+                                Cfixed=Cfixed,Dfixed=Dfixed,Bstart=Bstart,Cstart=Cstart,
+                                Dstart=Dstart,Bstruc=Bstruc,Cstruc=Cstruc,Dstruc=Dstruc)
+        } # end if(missingdata)
       } else {
         pfaclist <- vector("list",nstart)
-        for(j in 1:nstart){
-          pfaclist[[j]] <- parafac_4way(data=X,nfac=nfac,xcx=xcx,const=const,control=control,
-                                        maxit=maxit,ctol=ctol,Bfixed=Bfixed,Cfixed=Cfixed,
-                                        Dfixed=Dfixed,Bstart=Bstart,Cstart=Cstart,Dstart=Dstart,
-                                        Bstruc=Bstruc,Cstruc=Cstruc,Dstruc=Dstruc)
-        }
+        if(verbose) pbar <- txtProgressBar(min = 0, max = nstart, style = 3)
+        if(missingdata){
+          for(j in 1:nstart){
+            pfaclist[[j]] <- parafac_4wayna(data=X,nfac=nfac,naid=naid,const=const,control=control,
+                                            maxit=maxit,ctol=ctol,Bfixed=Bfixed,Cfixed=Cfixed,
+                                            Dfixed=Dfixed,Bstart=Bstart,Cstart=Cstart,Dstart=Dstart,
+                                            Bstruc=Bstruc,Cstruc=Cstruc,Dstruc=Dstruc)
+            if(verbose) setTxtProgressBar(pbar, j)
+          }
+        } else {
+          for(j in 1:nstart){
+            pfaclist[[j]] <- parafac_4way(data=X,nfac=nfac,xcx=xcx,const=const,control=control,
+                                          maxit=maxit,ctol=ctol,Bfixed=Bfixed,Cfixed=Cfixed,
+                                          Dfixed=Dfixed,Bstart=Bstart,Cstart=Cstart,Dstart=Dstart,
+                                          Bstruc=Bstruc,Cstruc=Cstruc,Dstruc=Dstruc)
+            if(verbose) setTxtProgressBar(pbar, j)
+          }
+        } # end if(missingdata)
+        if(verbose) close(pbar)
       } # end if(parallel)
     } # end if(lxdim==3L)
     
     # output results
     if(output[1]=="best"){
       cflag <- sapply(pfaclist,function(x) x$cflag)
-      Rsq <- sapply(pfaclist,function(x) x$Rsq)
+      SSE <- sapply(pfaclist,function(x) x$SSE)
+      #Rsq <- sapply(pfaclist,function(x) x$Rsq)
       cidx <- which(cflag==2)
       nbad <- length(cidx)
       if(nbad > 0 & nbad < nstart){
-        Rsq[cidx] <- 0
+        #Rsq[cidx] <- 0
+        SSE[cidx] <- max(SSE) + 1
       }
-      pfac <- pfaclist[[which.max(Rsq)]]
+      #pfac <- pfaclist[[which.max(Rsq)]]
+      pfac <- pfaclist[[which.min(SSE)]]
       class(pfac) <- "parafac"
       return(pfac)
     } else {
