@@ -1,35 +1,26 @@
-parafac2_4way <- 
-  function(data,nfac,xcx=sumsq(data),const=rep(0L,4),
+parafac2_3way <- 
+  function(data,nfac,xcx=sumsq(data),const=rep(0L,3),
            maxit=500,ctol=1e-4,Gfixed=NULL,Bfixed=NULL,
-           Cfixed=NULL,Dfixed=NULL,Gstart=NULL,Bstart=NULL,
-           Cstart=NULL,Dstart=NULL,Gstruc=NULL,Bstruc=NULL,
-           Cstruc=NULL,Dstruc=NULL,control=const.control(const)){
-    # 4-way Parallel Factor Analysis 2 (Parafac2)
+           Cfixed=NULL,Gstart=NULL,Bstart=NULL,Cstart=NULL,
+           Gstruc=NULL,Bstruc=NULL,Cstruc=NULL,
+           control=const.control(const)){
+    # 3-way Parallel Factor Analysis 2 (Parafac2)
     # Nathaniel E. Helwig (helwig@umn.edu)
-    # last updated: May 16, 2017
+    # last updated: September 5, 2017
     
     ### initialize Khatri-Rao product matrices
-    xdims <- rep(NA,4)
-    xdims[2] <- dim(data[[1]])[2]
-    xdims[3] <- dim(data[[1]])[3]
-    xdims[4] <- length(data)
-    if(is.null(Dfixed)){CBkrA <- matrix(0,nfac*xdims[2]*xdims[3],nfac)}
-    if(is.null(Cfixed)){DBkrA <- matrix(0,nfac*xdims[2]*xdims[4],nfac)}
-    if(is.null(Bfixed)){DCkrA <- matrix(0,nfac*xdims[3]*xdims[4],nfac)}
-    DCkrB <- matrix(0,xdims[2]*xdims[3]*xdims[4],nfac)
+    xdims <- rep(NA,3)
+    xdims[2] <- ncol(data[[1]])
+    xdims[3] <- length(data)
+    if(is.null(Cfixed)){BkrA <- matrix(0,nfac*xdims[2],nfac)}
+    if(is.null(Bfixed)){CkrA <- matrix(0,nfac*xdims[3],nfac)}
     CkrB <- matrix(0,xdims[2]*xdims[3],nfac)
     
     ### initialize stuff for Mode A update
-    Rknew <- vector("list",xdims[4])
-    Xtilde <- array(0,dim=c(nfac,xdims[2],xdims[3],xdims[4]))
+    Rknew <- vector("list",xdims[3])
+    Xtilde <- array(0,dim=c(nfac,xdims[2],xdims[3]))
     
-    ### reshape raw data
-    for(kk in 1:xdims[4]){
-      mdim <- dim(data[[kk]])
-      data[[kk]] <- matrix(data[[kk]],mdim[1],xdims[2]*xdims[3])
-    }
-    
-    ### check for smoothness on Mode A
+    ### check for periodicity and smoothness on Mode A
     nx <- sapply(data, nrow)
     if(const[1]==5L){
       if(min(nx) == max(nx)){
@@ -37,18 +28,17 @@ parafac2_4way <-
         Ra <- tcrossprod(svd(cbind(1,MsplineBasis(1:nx[1], df=control$df[1], degree=control$degree[1], intercept=FALSE)$X[,-control$df[1]]))$u)
       } else {
         SAMEMODEA <- FALSE
-        Ra <- vector("list",xdims[4])
-        for(k in 1:xdims[4]) Ra[[k]] <- tcrossprod(svd(cbind(1,MsplineBasis(1:nx[k], df=control$df[1], degree=control$degree[1], intercept=FALSE)$X[,-control$df[1]]))$u)
+        Ra <- vector("list",xdims[3])
+        for(k in 1:xdims[3]) Ra[[k]] <- tcrossprod(svd(cbind(1,MsplineBasis(1:nx[k], df=control$df[1], degree=control$degree[1], intercept=FALSE)$X[,-control$df[1]]))$u)
       }
-    }
-    if(const[1]==6L){
+    } else if(const[1]==6L){
       if(min(nx) == max(nx)){
         SAMEMODEA <- TRUE
         Ra <- tcrossprod(svd(MsplineBasis(1:nx[1], df=control$df[1], degree=control$degree[1], intercept=TRUE)$X)$u)
       } else {
         SAMEMODEA <- FALSE
-        Ra <- vector("list",xdims[4])
-        for(k in 1:xdims[4]) Ra[[k]] <- tcrossprod(svd(MsplineBasis(1:nx[k], df=control$df[1], degree=control$degree[1], intercept=TRUE)$X)$u)
+        Ra <- vector("list",xdims[3])
+        for(k in 1:xdims[3]) Ra[[k]] <- tcrossprod(svd(MsplineBasis(1:nx[k], df=control$df[1], degree=control$degree[1], intercept=TRUE)$X)$u)
       }
     }
     
@@ -88,7 +78,7 @@ parafac2_4way <-
           Rb <- tcrossprod(svd(cbind(1,MsplineBasis(1:xdims[2], df=control$df[2], degree=control$degree[2], intercept=FALSE)$X[,-control$df[2]]))$u)
           Bdf <- rep(control$df[2], nfac)
         }
-      } else if(const[2]==6L){
+      } else if (const[2]==6L){
         Bold <- Bnew <- replicate(nfac, rfunc(1:xdims[2], df=control$df[2], degree=control$degree[2], type="smo", nonneg=control$nonneg[2]))
         Bdf <- rep(0, nfac)
         if(!control$nonneg[2]) {
@@ -134,41 +124,6 @@ parafac2_4way <-
     } else {
       Cold <- Cnew <- Cfixed
     }
-    if(is.null(Dfixed)){
-      if(!is.null(Dstart)){
-        Dold <- Dnew <- Dstart
-      } else if(const[4]==0L){
-        #Dold <- matrix(rnorm(xdims[4]*nfac),xdims[4],nfac)
-        Dold <- matrix(runif(xdims[4]*nfac),xdims[4],nfac)
-      } else if(const[4]==1L){
-        Dold <- svd(matrix(rnorm(xdims[4]*nfac),xdims[4],nfac),nu=nfac,nv=0)$u
-      } else if(const[4]==2L){
-        Dold <- Dnew <- matrix(runif(xdims[4]*nfac),xdims[4],nfac)
-      } else if(const[4]==3L){
-        Dold <- Dnew <- replicate(nfac, rfunc(1:xdims[4], df=control$df[4], degree=control$degree[4], type="uni", nonneg=control$nonneg[4]))
-        Ddf <- rep(0, nfac)
-      } else if(const[4]==4L){
-        Dold <- Dnew <- replicate(nfac, rfunc(1:xdims[4], df=control$df[4], degree=control$degree[4], type="mon", nonneg=control$nonneg[4]))
-        Ddf <- rep(0, nfac)
-      } else if(const[4]==5L){
-        Dold <- Dnew <- replicate(nfac, rfunc(1:xdims[4], df=control$df[4], degree=control$degree[4], type="per", nonneg=control$nonneg[4]))
-        Ddf <- rep(0, nfac)
-        if(!control$nonneg[4]) {
-          Rd <- tcrossprod(svd(cbind(1,MsplineBasis(1:xdims[4], df=control$df[4], degree=control$degree[4], intercept=FALSE)$X[,-control$df[4]]))$u)
-          Ddf <- rep(control$df[4], nfac)
-        }
-      } else if(const[4]==6L){
-        Dold <- Dnew <- replicate(nfac, rfunc(1:xdims[4], df=control$df[4], degree=control$degree[4], type="smo", nonneg=control$nonneg[4]))
-        Ddf <- rep(0, nfac)
-        if(!control$nonneg[4]) {
-          Rd <- tcrossprod(svd(MsplineBasis(1:xdims[4], df=control$df[4], degree=control$degree[4], intercept=TRUE)$X)$u)
-          Ddf <- rep(control$df[4], nfac)
-        }
-      }
-      if(!is.null(Dstruc)) Dold <- Dnew <- Dold * Dstruc
-    } else {
-      Dold <- Dnew <- Dfixed
-    }
     
     ### iterative update of matrices
     vtol <- sseold <- xcx
@@ -178,62 +133,60 @@ parafac2_4way <-
       
       ## Step 1: update mode A weights
       # 1a: update orthogonal projections
-      for(u in 1:nfac){CkrB[,u] <- kronecker(Cold[,u],Bold[,u])}
       if(any(const[1]==c(5L,6L))){
         if(SAMEMODEA){
-          for(kk in 1:xdims[4]){
-            xsvd <- svd(Ra%*%data[[kk]]%*%CkrB%*%tcrossprod((diag(nfac)*Dold[kk,]),Gold))
+          for(kk in 1:xdims[3]){
+            xsvd <- svd(Ra%*%data[[kk]]%*%Bold%*%tcrossprod((diag(nfac)*Cold[kk,]),Gold))
             Rknew[[kk]] <- tcrossprod(xsvd$u,xsvd$v)
-            Xtilde[,,,kk] <- array(crossprod(Rknew[[kk]],data[[kk]]),dim=c(nfac,xdims[2],xdims[3]))
+            Xtilde[,,kk] <- crossprod(Rknew[[kk]],data[[kk]])
           }
         } else {
-          for(kk in 1:xdims[4]){
-            xsvd <- svd(Ra[[kk]]%*%data[[kk]]%*%CkrB%*%tcrossprod((diag(nfac)*Dold[kk,]),Gold))
+          for(kk in 1:xdims[3]){
+            xsvd <- svd(Ra[[kk]]%*%data[[kk]]%*%Bold%*%tcrossprod((diag(nfac)*Cold[kk,]),Gold))
             Rknew[[kk]] <- tcrossprod(xsvd$u,xsvd$v)
-            Xtilde[,,,kk] <- array(crossprod(Rknew[[kk]],data[[kk]]),dim=c(nfac,xdims[2],xdims[3]))
+            Xtilde[,,kk] <- crossprod(Rknew[[kk]],data[[kk]])
           }
         }
       } else {
-        for(kk in 1:xdims[4]){
-          xsvd <- svd(data[[kk]]%*%CkrB%*%tcrossprod((diag(nfac)*Dold[kk,]),Gold))
+        for(kk in 1:xdims[3]){
+          xsvd <- svd(data[[kk]]%*%Bold%*%tcrossprod((diag(nfac)*Cold[kk,]),Gold))
           Rknew[[kk]] <- tcrossprod(xsvd$u,xsvd$v)
-          Xtilde[,,,kk] <- array(crossprod(Rknew[[kk]],data[[kk]]),dim=c(nfac,xdims[2],xdims[3]))
+          Xtilde[,,kk] <- crossprod(Rknew[[kk]],data[[kk]])
         }
       }
-      
       # 1b: update correlation matrix
       if(is.null(Gfixed)){
         if(is.null(Gstruc)){
           if(const[1]!=1L){
-            Xa <- matrix(Xtilde,nfac,xdims[2]*xdims[3]*xdims[4])
-            for(u in 1:nfac){DCkrB[,u] <- kronecker(Dold[,u],kronecker(Cold[,u],Bold[,u]))}
-            #Gnew <- Xa%*%DCkrB%*%smpower(crossprod(DCkrB),-1)
-            Gnew <- Xa%*%DCkrB%*%smpower(crossprod(Dold)*crossprod(Cold)*crossprod(Bold),-1)
+            Xa <- matrix(Xtilde,nfac,xdims[2]*xdims[3])
+            for(u in 1:nfac){CkrB[,u] <- kronecker(Cold[,u],Bold[,u])}
+            #Gnew <- Xa%*%CkrB%*%smpower(crossprod(CkrB),-1)
+            Gnew <- Xa%*%CkrB%*%smpower(crossprod(Cold)*crossprod(Bold),-1)
           }
         } else {
-          Xa <- matrix(Xtilde,nfac,xdims[2]*xdims[3]*xdims[4])
-          for(u in 1:nfac){DCkrB[,u] <- kronecker(Dold[,u],kronecker(Cold[,u],Bold[,u]))}
+          Xa <- matrix(Xtilde,nfac,xdims[2]*xdims[3])
+          for(u in 1:nfac){CkrB[,u] <- kronecker(Cold[,u],Bold[,u])}
           for(u in 1:nfac){
-            Zhat <- Xa - tcrossprod(Gnew[,-u],DCkrB[,-u])
-            Gnew[,u] <- ( (Zhat %*% DCkrB[,u]) / sum(DCkrB[,u]^2) ) * Gstruc[,u]
+            Zhat <- Xa - tcrossprod(Gnew[,-u],CkrB[,-u])
+            Gnew[,u] <- ( (Zhat %*% CkrB[,u]) / sum(CkrB[,u]^2) ) * Gstruc[,u]
           }
         } # end if(is.null(Gstruc))
       } # end if(is.null(Gfixed))
       
       ## Step 2: update mode B weights
       if(is.null(Bfixed)){
-        Xb <- matrix(aperm(Xtilde,perm=c(2,1,3,4)),xdims[2],nfac*xdims[3]*xdims[4])
-        for(u in 1:nfac){DCkrA[,u] <- kronecker(Dold[,u],kronecker(Cold[,u],Gnew[,u]))}
+        Xb <- matrix(aperm(Xtilde,perm=c(2,1,3)),xdims[2],nfac*xdims[3])
+        for(u in 1:nfac){CkrA[,u] <- kronecker(Cold[,u],Gnew[,u])}
         if(is.null(Bstruc)){
           if(const[2]==0L){
-            #Bnew <- Xb%*%DCkrA%*%smpower(crossprod(DCkrA),-1)
-            Bnew <- Xb%*%DCkrA%*%smpower(crossprod(Dold)*crossprod(Cold)*crossprod(Gnew),-1)
+            #Bnew <- Xb%*%CkrA%*%smpower(crossprod(CkrA),-1)
+            Bnew <- Xb%*%CkrA%*%smpower(crossprod(Cold)*crossprod(Gnew),-1)
           } else if(const[2]==1L) {
-            Zmat <- Xb%*%DCkrA
+            Zmat <- Xb%*%CkrA
             Bnew <- Zmat%*%smpower(crossprod(Zmat),-0.5)
           } else if(const[2]==2L) {
-            cpmat <- crossprod(DCkrA)
-            for(ii in 1:xdims[2]){Bnew[ii,] <- fnnls(cpmat,crossprod(DCkrA,Xb[ii,]))}
+            cpmat <- crossprod(CkrA)
+            for(ii in 1:xdims[2]){Bnew[ii,] <- fnnls(cpmat,crossprod(CkrA,Xb[ii,]))}
             zix <- which(Bnew <= 1e-9)
             if(length(zix) > 0) Bnew[zix] <- 0
             if(any(colSums(Bnew)==0)){
@@ -243,8 +196,8 @@ parafac2_4way <-
             }
           } else if(const[2]==3L){
             for(u in 1:nfac){
-              Zhat <- Xb - tcrossprod(Bnew[,-u],DCkrA[,-u])
-              fit <- fumls(1:xdims[2], Zhat %*% DCkrA[,u] / sum(DCkrA[,u]^2), df=control$df[2], degree=control$degree[2], lower=ifelse(control$nonneg[2],0,-Inf))
+              Zhat <- Xb - tcrossprod(Bnew[,-u],CkrA[,-u])
+              fit <- fumls(1:xdims[2], Zhat %*% CkrA[,u] / sum(CkrA[,u]^2), df=control$df[2], degree=control$degree[2], lower=ifelse(control$nonneg[2],0,-Inf))
               Bdf[u] <- fit$edf
               Bnew[,u] <- fit$fitted.values
             }
@@ -257,8 +210,8 @@ parafac2_4way <-
             }
           } else if(const[2]==4L){
             for(u in 1:nfac){
-              Zhat <- Xb - tcrossprod(Bnew[,-u],DCkrA[,-u])
-              fit <- fmnls(1:xdims[2], Zhat %*% DCkrA[,u] / sum(DCkrA[,u]^2), df=control$df[2], degree=control$degree[2], lower=ifelse(control$nonneg[2],0,-Inf))
+              Zhat <- Xb - tcrossprod(Bnew[,-u],CkrA[,-u])
+              fit <- fmnls(1:xdims[2], Zhat %*% CkrA[,u] / sum(CkrA[,u]^2), df=control$df[2], degree=control$degree[2], lower=ifelse(control$nonneg[2],0,-Inf))
               Bdf[u] <- fit$edf
               Bnew[,u] <- fit$fitted.values
             }
@@ -271,13 +224,13 @@ parafac2_4way <-
             }
           } else if(const[2]==5L | const[2]==6L){
             for(u in 1:nfac){
-              Zhat <- Xb - tcrossprod(Bnew[,-u],DCkrA[,-u])
+              Zhat <- Xb - tcrossprod(Bnew[,-u],CkrA[,-u])
               if(control$nonneg[2]){
-                fit <- fsmls(1:xdims[2], Zhat %*% DCkrA[,u] / sum(DCkrA[,u]^2), df=control$df[2], degree=control$degree[2], nonneg=TRUE, periodic=ifelse(const[2]==5L,TRUE,FALSE))
+                fit <- fsmls(1:xdims[2], Zhat %*% CkrA[,u] / sum(CkrA[,u]^2), df=control$df[2], degree=control$degree[2], nonneg=TRUE, periodic=ifelse(const[2]==5L,TRUE,FALSE))
                 Bdf[u] <- fit$edf
                 Bnew[,u] <- fit$fitted.values
               } else {
-                Bnew[,u] <- Rb %*% Zhat %*% DCkrA[,u] / sum(DCkrA[,u]^2)
+                Bnew[,u] <- Rb %*% Zhat %*% CkrA[,u] / sum(CkrA[,u]^2)
               }
             }
             if(control$nonneg[2]){
@@ -292,26 +245,26 @@ parafac2_4way <-
           } # end if(const[2]==0L)
         } else {
           for(u in 1:nfac){
-            Zhat <- Xb - tcrossprod(Bnew[,-u],DCkrA[,-u])
-            Bnew[,u] <- ( (Zhat %*% DCkrA[,u]) / sum(DCkrA[,u]^2) ) * Bstruc[,u]
+            Zhat <- Xb - tcrossprod(Bnew[,-u],CkrA[,-u])
+            Bnew[,u] <- ( (Zhat %*% CkrA[,u]) / sum(CkrA[,u]^2) ) * Bstruc[,u]
           }
         } # end if(is.null(Bstruc))
       } # end if(is.null(Bfixed))
       
       ## Step 3: update mode C weights
       if(is.null(Cfixed)){
-        Xc <- matrix(aperm(Xtilde,perm=c(3,1,2,4)),xdims[3],nfac*xdims[2]*xdims[4])
-        for(u in 1:nfac){DBkrA[,u] <- kronecker(Dold[,u],kronecker(Bnew[,u],Gnew[,u]))}
+        Xc <- matrix(aperm(Xtilde,perm=c(3,1,2)),xdims[3],nfac*xdims[2])
+        for(u in 1:nfac){BkrA[,u] <- kronecker(Bnew[,u],Gnew[,u])}
         if(is.null(Cstruc)){
           if(const[3]==0L){
-            #Cnew <- Xc%*%DBkrA%*%smpower(crossprod(DBkrA),-1)
-            Cnew <- Xc%*%DBkrA%*%smpower(crossprod(Dold)*crossprod(Bnew)*crossprod(Gnew),-1)
+            #Cnew <- Xc%*%BkrA%*%smpower(crossprod(BkrA),-1)
+            Cnew <- Xc%*%BkrA%*%smpower(crossprod(Bnew)*crossprod(Gnew),-1)
           } else if(const[3]==1L) {
-            Zmat <- Xc%*%DBkrA
+            Zmat <- Xc%*%BkrA
             Cnew <- Zmat%*%smpower(crossprod(Zmat),-0.5)
           } else if(const[3]==2L) {
-            cpmat <- crossprod(DBkrA)
-            for(ii in 1:xdims[3]){Cnew[ii,] <- fnnls(cpmat,crossprod(DBkrA,Xc[ii,]))}
+            cpmat <- crossprod(BkrA)
+            for(ii in 1:xdims[3]){Cnew[ii,] <- fnnls(cpmat,crossprod(BkrA,Xc[ii,]))}
             zix <- which(Cnew <= 1e-9)
             if(length(zix) > 0) Cnew[zix] <- 0
             if(any(colSums(Cnew)==0)){
@@ -321,8 +274,8 @@ parafac2_4way <-
             }
           } else if(const[3]==3L){
             for(u in 1:nfac){
-              Zhat <- Xc - tcrossprod(Cnew[,-u],DBkrA[,-u])
-              fit <- fumls(1:xdims[3], Zhat %*% DBkrA[,u] / sum(DBkrA[,u]^2), df=control$df[3], degree=control$degree[3], lower=ifelse(control$nonneg[3],0,-Inf))
+              Zhat <- Xc - tcrossprod(Cnew[,-u],BkrA[,-u])
+              fit <- fumls(1:xdims[3], Zhat %*% BkrA[,u] / sum(BkrA[,u]^2), df=control$df[3], degree=control$degree[3], lower=ifelse(control$nonneg[3],0,-Inf))
               Cdf[u] <- fit$edf
               Cnew[,u] <- fit$fitted.values
             }
@@ -335,8 +288,8 @@ parafac2_4way <-
             }
           } else if(const[3]==4L){
             for(u in 1:nfac){
-              Zhat <- Xc - tcrossprod(Cnew[,-u],DBkrA[,-u])
-              fit <- fmnls(1:xdims[3], Zhat %*% DBkrA[,u] / sum(DBkrA[,u]^2), df=control$df[3], degree=control$degree[3], lower=ifelse(control$nonneg[3],0,-Inf))
+              Zhat <- Xc - tcrossprod(Cnew[,-u],BkrA[,-u])
+              fit <- fmnls(1:xdims[3], Zhat %*% BkrA[,u] / sum(BkrA[,u]^2), df=control$df[3], degree=control$degree[3], lower=ifelse(control$nonneg[3],0,-Inf))
               Cdf[u] <- fit$edf
               Cnew[,u] <- fit$fitted.values
             }
@@ -349,13 +302,13 @@ parafac2_4way <-
             }
           } else if(const[3]==5L | const[3]==6L){
             for(u in 1:nfac){
-              Zhat <- Xc - tcrossprod(Cnew[,-u],DBkrA[,-u])
+              Zhat <- Xc - tcrossprod(Cnew[,-u],BkrA[,-u])
               if(control$nonneg[3]){
-                fit <- fsmls(1:xdims[3], Zhat %*% DBkrA[,u] / sum(DBkrA[,u]^2), df=control$df[3], degree=control$degree[3], nonneg=TRUE, periodic=ifelse(const[3]==5L,TRUE,FALSE))
+                fit <- fsmls(1:xdims[3], Zhat %*% BkrA[,u] / sum(BkrA[,u]^2), df=control$df[3], degree=control$degree[3], nonneg=TRUE, periodic=ifelse(const[3]==5L,TRUE,FALSE))
                 Cdf[u] <- fit$edf
                 Cnew[,u] <- fit$fitted.values
               } else {
-                Cnew[,u] <- Rc %*% Zhat %*% DBkrA[,u] / sum(DBkrA[,u]^2)
+                Cnew[,u] <- Rc %*% Zhat %*% BkrA[,u] / sum(BkrA[,u]^2)
               }
             }
             if(control$nonneg[3]){
@@ -370,117 +323,43 @@ parafac2_4way <-
           } # end if(const[3]==0L)
         } else {
           for(u in 1:nfac){
-            Zhat <- Xc - tcrossprod(Cnew[,-u],DBkrA[,-u])
-            Cnew[,u] <- ( (Zhat %*% DBkrA[,u]) / sum(DBkrA[,u]^2) ) * Cstruc[,u]
+            Zhat <- Xc - tcrossprod(Cnew[,-u],BkrA[,-u])
+            Cnew[,u] <- ( (Zhat %*% BkrA[,u]) / sum(BkrA[,u]^2) ) * Cstruc[,u]
           }
         } # end if(is.null(Cstruc))
       } # end if(is.null(Cfixed))
       
-      ## Step 4: update mode D weights
-      if(is.null(Dfixed)){
-        Xd <- matrix(aperm(Xtilde,perm=c(4,1,2,3)),xdims[4],nfac*xdims[2]*xdims[3])
-        for(u in 1:nfac){CBkrA[,u] <- kronecker(Cnew[,u],kronecker(Bnew[,u],Gnew[,u]))}
-        if(is.null(Dstruc)){
-          if(const[4]==0L){
-            #Dnew <- Xd%*%CBkrA%*%smpower(crossprod(CBkrA),-1)
-            Dnew <- Xd%*%CBkrA%*%smpower(crossprod(Cnew)*crossprod(Bnew)*crossprod(Gnew),-1)
-          } else if(const[4]==1L) {
-            Zmat <- Xd%*%CBkrA
-            Dnew <- Zmat%*%smpower(crossprod(Zmat),-0.5)
-          } else if(const[4]==2L) {
-            cpmat <- crossprod(CBkrA)
-            for(ii in 1:xdims[4]){Dnew[ii,] <- fnnls(cpmat,crossprod(CBkrA,Xd[ii,]))}
-            zix <- which(Dnew <= 1e-9)
-            if(length(zix) > 0) Dnew[zix] <- 0
-            if(any(colSums(Dnew)==0)){
-              Dnew <- Dold
-              vtol <- 0
-              cflag <- 2
-            }
-          } else if(const[4]==3L){
-            for(u in 1:nfac){
-              Zhat <- Xd - tcrossprod(Dnew[,-u],CBkrA[,-u])
-              fit <- fumls(1:xdims[4], Zhat %*% CBkrA[,u] / sum(CBkrA[,u]^2), df=control$df[4], degree=control$degree[4], lower=ifelse(control$nonneg[4],0,-Inf))
-              Ddf[u] <- fit$edf
-              Dnew[,u] <- fit$fitted.values
-            }
-            zix <- which(Dnew <= 1e-9)
-            if(length(zix) > 0) Dnew[zix] <- 0
-            if(any(colSums(Dnew)==0)){
-              Dnew <- Dold
-              vtol <- 0
-              cflag <- 2
-            }
-          } else if(const[4]==4L){
-            for(u in 1:nfac){
-              Zhat <- Xd - tcrossprod(Dnew[,-u],CBkrA[,-u])
-              fit <- fmnls(1:xdims[4], Zhat %*% CBkrA[,u] / sum(CBkrA[,u]^2), df=control$df[4], degree=control$degree[4], lower=ifelse(control$nonneg[4],0,-Inf))
-              Ddf[u] <- fit$edf
-              Dnew[,u] <- fit$fitted.values
-            }
-            zix <- which(Dnew <= 1e-9)
-            if(length(zix) > 0) Dnew[zix] <- 0
-            if(any(colSums(Dnew)==0)){
-              Dnew <- Dold
-              vtol <- 0
-              cflag <- 2
-            }
-          } else if(const[4]==5L | const[4]==6L){
-            for(u in 1:nfac){
-              Zhat <- Xd - tcrossprod(Dnew[,-u],CBkrA[,-u])
-              if(control$nonneg[4]){
-                fit <- fsmls(1:xdims[4], Zhat %*% CBkrA[,u] / sum(CBkrA[,u]^2), df=control$df[4], degree=control$degree[4], nonneg=TRUE, periodic=ifelse(const[4]==5L,TRUE,FALSE))
-                Ddf[u] <- fit$edf
-                Dnew[,u] <- fit$fitted.values
-              } else {
-                Dnew[,u] <- Rd %*% Zhat %*% CBkrA[,u] / sum(CBkrA[,u]^2)
-              }
-            }
-          } # end if(const[3]==0L)
-        } else {
-          for(u in 1:nfac){
-            Zhat <- Xd - tcrossprod(Dnew[,-u],CBkrA[,-u])
-            Dnew[,u] <- ( (Zhat %*% CBkrA[,u]) / sum(CBkrA[,u]^2) ) * Dstruc[,u]
-          }
-        } # end if(is.null(Dstruc))
-      } # end if(is.null(Dfixed))
-      
-      ## Step 5: check for convergence
-      for(u in 1:nfac){CkrB[,u] <- kronecker(Cnew[,u],Bnew[,u])}
+      ## Step 4: check for convergence
       ssenew <- 0
-      for(kk in 1:xdims[4]){
-        ssenew <- ssenew + sum((data[[kk]]-tcrossprod(Rknew[[kk]]%*%Gnew%*%(diag(nfac)*Dnew[kk,]),CkrB))^2)
+      for(kk in 1:xdims[3]){
+        ssenew <- ssenew + sum((data[[kk]]-tcrossprod(Rknew[[kk]]%*%Gnew%*%(diag(nfac)*Cnew[kk,]),Bnew))^2)
       }
       #vtol <- (sseold-ssenew)/sseold
       vtol <- (sseold - ssenew) / xcx
       Gold <- Gnew
       Bold <- Bnew
       Cold <- Cnew
-      Dold <- Dnew
       sseold <- ssenew
       iter <- iter + 1
       
     }  # end while(vtol>ctol && iter<maxit)
     
     ### scale and order solution
-    if(is.null(Gfixed) & is.null(Bfixed) & is.null(Cfixed) & is.null(Dfixed)){
+    if(is.null(Gfixed) & is.null(Bfixed) & is.null(Cfixed)){
       
-      # put the scale in Mode D
+      # put the scale in Mode C
       adg <- colSums(Gnew^2)
       Gnew <- Gnew%*%(diag(nfac)*(adg^-0.5))
       bdg <- colMeans(Bnew^2)
       Bnew <- Bnew%*%(diag(nfac)*(bdg^-0.5))
-      cdg <- colMeans(Cnew^2)
-      Cnew <- Cnew%*%(diag(nfac)*(cdg^-0.5))
-      Dnew <- Dnew%*%(diag(nfac)*((adg*bdg*cdg)^0.5))
+      Cnew <- Cnew%*%(diag(nfac)*((adg*bdg)^0.5))
       
       # order according to sum-of-squares
-      if(is.null(Gstruc) & is.null(Bstruc) & is.null(Cstruc) & is.null(Dstruc)){
-        fordr <- order(colSums(Dnew^2),decreasing=TRUE)
+      if(is.null(Gstruc) & is.null(Bstruc) & is.null(Cstruc)){
+        fordr <- order(colSums(Cnew^2),decreasing=TRUE)
         Gnew <- as.matrix(Gnew[,fordr])
         Bnew <- as.matrix(Bnew[,fordr])
         Cnew <- as.matrix(Cnew[,fordr])
-        Dnew <- as.matrix(Dnew[,fordr])
       }
       
     }
@@ -490,8 +369,8 @@ parafac2_4way <-
       ntotal <- xdims[3] * control$df[1]
     } else {
       ntotal <- sum(nx)
-    } # end if(any(const[1]==c(5L,6L)))
-    Adf <- nfac * (ntotal - xdims[4]*(nfac+1)/2)
+    } # end if(const[1]==5L)
+    Adf <- nfac * (ntotal - xdims[3]*(nfac+1)/2)
     
     ### effective degrees of freedom (Mode A; crossproducts)
     if(is.null(Gfixed)){
@@ -532,39 +411,20 @@ parafac2_4way <-
     if(is.null(Cfixed)){
       if(is.null(Cstruc)){
         if(const[3]==0L){
-          Cdf <- nfac * (xdims[3] - 1L)
+          Cdf <- nfac * xdims[3]
         } else if(const[3]==1L){
-          Cdf <- nfac * (xdims[3] - (nfac+1)/2)
+          Cdf <- nfac * (xdims[3] - (nfac-1)/2)
         } else if(const[3]==2L){
-          Cdf <- nfac * (xdims[3] - 1L) - sum(Cnew==0)
+          Cdf <- nfac * xdims[3] - sum(Cnew==0)
         } else if(const[3]>=3L & const[3]<=6L){
-          Cdf <- sum(Cdf) - nfac
-        }
+          Cdf <- sum(Cdf)
+        } 
       } else {
-        Cdf <- sum(Cstruc) - nfac
+        Cdf <- sum(Cstruc)
       } # end if(is.null(Cstruc))
     } else {
       Cdf <- 0
-    } # end if(is.null(Cfixed))
-    
-    ### effective degrees of freedom (Mode D)
-    if(is.null(Dfixed)){
-      if(is.null(Dstruc)){
-        if(const[4]==0L){
-          Ddf <- nfac * xdims[4]
-        } else if(const[4]==1L){
-          Ddf <- nfac * (xdims[4] - (nfac-1)/2)
-        } else if(const[4]==2L){
-          Ddf <- nfac * xdims[4] - sum(Dnew==0)
-        } else if(const[4]>=3L & const[4]<=6L){
-          Ddf <- sum(Ddf)
-        } 
-      } else {
-        Ddf <- sum(Dstruc)
-      } # end if(is.null(Cstruc))
-    } else {
-      Ddf <- 0
-      if(const[1]==1L){
+      if(const[1]==1L) {
         Bdf <- Bdf + nfac
       } else {
         Gdf <- Gdf + nfac
@@ -572,8 +432,8 @@ parafac2_4way <-
     } # end if(is.null(Cfixed))
     
     ### GCV criterion
-    edf <- c(Adf+Gdf,Bdf,Cdf,Ddf)
-    pxdim <- sum(nx) * prod(xdims[2:3])
+    edf <- c(Adf+Gdf,Bdf,Cdf)
+    pxdim <- sum(nx) * xdims[2]
     GCV <- (ssenew/pxdim) / (1 - sum(edf)/pxdim)^2
     
     ### collect results
@@ -581,14 +441,12 @@ parafac2_4way <-
     if(is.na(cflag)){
       if(vtol<=ctol){cflag <- 0} else {cflag <- 1}
     }
-    names(edf) <- c("A","B","C","D")
-    fixed <- c(ifelse(is.null(Gfixed), FALSE, TRUE), ifelse(is.null(Bfixed), FALSE, TRUE), 
-               ifelse(is.null(Cfixed), FALSE, TRUE), ifelse(is.null(Dfixed), FALSE, TRUE))
-    struc <- c(ifelse(is.null(Gstruc), FALSE, TRUE), ifelse(is.null(Bstruc), FALSE, TRUE), 
-               ifelse(is.null(Cstruc), FALSE, TRUE), ifelse(is.null(Dstruc), FALSE, TRUE))
-    Ak <- vector("list", xdims[4])
-    for(k in 1:xdims[4]) Ak[[k]] <- Rknew[[k]] %*% Gnew
-    pfac <- list(A=Ak,B=Bnew,C=Cnew,D=Dnew,Phi=crossprod(Gnew),SSE=ssenew,Rsq=Rsq,
+    names(edf) <- c("A","B","C")
+    fixed <- c(ifelse(is.null(Gfixed), FALSE, TRUE), ifelse(is.null(Bfixed), FALSE, TRUE), ifelse(is.null(Cfixed), FALSE, TRUE))
+    struc <- c(ifelse(is.null(Gstruc), FALSE, TRUE), ifelse(is.null(Bstruc), FALSE, TRUE), ifelse(is.null(Cstruc), FALSE, TRUE))
+    Ak <- vector("list", xdims[3])
+    for(k in 1:xdims[3]) Ak[[k]] <- Rknew[[k]] %*% Gnew
+    pfac <- list(A=Ak,B=Bnew,C=Cnew,Phi=crossprod(Gnew),SSE=ssenew,Rsq=Rsq,
                  GCV=GCV,edf=edf,iter=iter,cflag=cflag,const=const,control=control,
                  fixed=fixed,struc=struc)
     return(pfac)
